% This file simulates DFT's of observations from factor models 
% with AR and MA loadings at the approximating frequencies.

% Outputs:  out1- matrix of DFT's of AR loadings data at 
%           frequences floor(theta)+2*pi*jjj/T with jjj=1:m
%           out2- matrix of DFT's of MA loadings data at
%           frequences floor(theta)+2*pi*jjj/T with jjj=1:m
%           out3- matrix of DFT's of AR loadings data at 
%           frequences floor(theta)+2*pi*(floor(T/2)-jjj)/T with jjj=1:m
%           out4- matrix of DFT's of MA loadings data at
%           frequences floor(theta)+2*pi*(floor(T/2)-jjj)/T with jjj=1:m

% Inputs: n-cross-sectional dimension
%         T-time series dimension
%         r-true number of factors
%         m-number of approximating frequencies
%         theta-the frequency of interest
%         ro-cross-sectional autoregressive parameter
%         distribution is 1 for Gaussian idiosyncratic terms
%                      2 for chi(2)-1 idiosyncratic terms
%                      and 3 for t(5) idiosyncratic terms

% December 30, 2007
% Set parameters
function [out1,out2,out3,out4]=simulation_figure2_final(n,T,r,m,theta,ro,distribution)
% First, simulate the systematic part of the data
% CASE 1: AR loadings
b0=randn(r*n,1);
b1=0.1*rand(r*n,1)+.8;
b2=0.1*rand(r*n,1)+.5;
% Initialize
SYST=zeros(n,T);
Normaliz=((1+b1.*b2).*b0.^2)./((1-b2.^2).*(1-b1.^2).*(1-b1.*b2));% this is variance of the AR(2)
Norma=zeros(n,1);
for ii=1:r
    syst=zeros(n,2);
    for index=1:(T-2)
        syst(:,2+index)=b0((n*(ii-1)+1):(n*ii),1)*randn(1,1)+(b1((n*(ii-1)+1):(n*ii),1)+...
            b2((n*(ii-1)+1):(n*ii),1)).*syst(:,1+index)-...
            (b1((n*(ii-1)+1):(n*ii),1).*b2((n*(ii-1)+1):(n*ii),1)).*syst(:,index);
    end
    SYST=SYST+syst;
    Norma=Norma+Normaliz((n*(ii-1)+1):(n*ii),1);
end
syst_norm_AR=sqrt(.4+.05*r)*diag(Norma.^(-0.5))*SYST;

% CASE II: MA loadings
b0=randn(r*n,1);
b1=-rand(r*n,1);
b2=-rand(r*n,1);
Normaliz=(b0.^2).*(1+(b1+b2).^2+(b1.*b2).^2);% this is variance of the MA(2)
b1=[b0 -b0.*(b1+b2) b0.*b1.*b2];
factor=randn(r,T+2);
syst=kron(b1(:,1),ones(1,T)).*kron(factor(:,3:end),ones(n,1))+...
    kron(b1(:,2),ones(1,T)).*kron(factor(:,2:end-1),ones(n,1))+...
    kron(b1(:,3),ones(1,T)).*kron(factor(:,1:end-2),ones(n,1));
SYST=zeros(n,T);
NORMA=zeros(n,1);
for ii=1:r
    SYST=SYST+syst((n*(ii-1)+1):(n*ii),:);
    NORMA=NORMA+Normaliz((n*(ii-1)+1):(n*ii),:);
end
syst_norm_MA=sqrt(.4+.05*r)*diag(NORMA.^(-0.5))*SYST;

% Now generate the idiosyncratic terms
if distribution==1
v=randn(n,T);
elseif distribution==2
v=randn(n,T);
v=(v.*v-1)/sqrt(2);
else
v=sqrt(3/5)*(randn(n,T)./sqrt((randn(n,T).^2+randn(n,T).^2+randn(n,T).^2+randn(n,T).^2+randn(n,T).^2)/5));
end
er=v(1,:)/sqrt(1-ro^2);
for j=2:n
er(j,:)=ro*er(j-1,:)+v(j,:);
end
v=sqrt((1-ro^2))*er;%idiosincratic errors
rot=-0.8+1.6*rand(n,1);
er=v(:,1)./sqrt(1-rot.^2);
for j=2:T
er(:,j)=rot.*er(:,j-1)+v(:,j);
end
er=sqrt(.6-.05*r)*kron(sqrt((1-rot.^2)),ones(1,T)).*er;
idios_norm=er;
% Generate data
data1=syst_norm_AR+idios_norm;
data2=syst_norm_MA+idios_norm;
% Generate frequency domain data
fl=floor(T/2);
for jjj=1:m
    out1(:,jjj)=data1*(exp(-sqrt(-1)*(1:T)*(floor(theta)+2*pi*jjj/T)))';
    out2(:,jjj)=data2*(exp(-sqrt(-1)*(1:T)*(floor(theta)+2*pi*jjj/T)))';
    out3(:,jjj)=data1*(exp(-sqrt(-1)*(1:T)*(floor(theta)+2*pi*(fl-jjj)/T)))';
    out4(:,jjj)=data2*(exp(-sqrt(-1)*(1:T)*(floor(theta)+2*pi*(fl-jjj)/T)))';
end